@@ -1,4 +1,4 @@
-/* Copyright 2007-2011 ENSEIRB, INRIA & CNRS
+/* Copyright 2007-2012 IPB, Universite de Bordeaux, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -33,7 +33,7 @@
 /**                                                        **/
 /**   NAME       : bdgraph_bipart_ml.c                     **/
 /**                                                        **/
-/**   AUTHOR     : Jun-Ho HER                              **/
+/**   AUTHOR     : Jun-Ho HER (v6.0)                       **/
 /**                Francois PELLEGRINI                     **/
 /**                                                        **/
 /**   FUNCTION   : This module bipartitions a distributed  **/
@@ -41,6 +41,8 @@
 /**                                                        **/
 /**   DATES      : # Version 5.1  : from : 30 oct 2007     **/
 /**                                 to   : 14 apr 2011     **/
+/**              : # Version 6.0  : from : 11 sep 2011     **/
+/**                                 to   : 18 sep 2012     **/
 /**                                                        **/
 /************************************************************/
 
@@ -93,10 +95,13 @@ const BdgraphBipartMlParam * const    paraptr)    /*+ Method parameters
   else if (paraptr->duplvlmax < -1)               /* duplvlmax can allow only fold */
     foldval = (-(paraptr->duplvlmax + 1) < (finegrafptr->levlnum + 1)) ? 0 : -1;
 
+  *coarmultptr = NULL;                            /* Let the routine create the multinode array */
+  dgraphInit (&coargrafptr->s, finegrafptr->s.proccomm); /* Re-use fine graph communicator      */
   if (dgraphCoarsen (&finegrafptr->s, &coargrafptr->s, coarmultptr, paraptr->passnbr,
-                     paraptr->coarnbr, foldval, paraptr->dupmax, paraptr->coarrat) != 0)
+                     paraptr->coarnbr, 0, foldval, paraptr->dupmax, paraptr->coarrat) != 0)
     return (1);                                   /* Return if coarsening failed */
 
+  *coarmultptr -= coargrafptr->s.baseval;         /* Base pointer to multinode array    */
   coargrafptr->partgsttax = NULL;                 /* Do not allocate partition data yet */
   coargrafptr->fronloctab = NULL;
   coargrafptr->fronglbnbr = 0;
@@ -114,6 +119,7 @@ const BdgraphBipartMlParam * const    paraptr)    /*+ Method parameters
     if ((coarveexloctax = (Gnum *) memAlloc (coargrafptr->s.vertlocnbr * sizeof (Gnum))) == NULL) {
       errorPrint ("bdgraphBipartMlCoarsen: out of memory");
       dgraphExit (&coargrafptr->s);               /* Only free Dgraph since veexloctax not allocated */
+      memFree    (*coarmultptr + coargrafptr->s.baseval);
       return     (1);
     }
     coarveexloctax -= coargrafptr->s.baseval;
@@ -701,6 +707,7 @@ const DgraphCoarsenMulti * restrict const coarmulttax) /*+ Multinode array +*/
   finegrafptr->fronglbnbr      = reduglbtab[2];
   finegrafptr->commglbload     = ((reduglbtab[3] / 2) * finegrafptr->domdist + reduglbtab[4] + finegrafptr->commglbloadextn0);
   finegrafptr->commglbgainextn = reduglbtab[5];
+  finegrafptr->bbalglbval      = coargrafptr->bbalglbval;
 
   if (coargrafptr->s.procglbnbr != 0)
     memFree ((void *) (coarmulttax + coargrafptr->s.baseval));
