@@ -35,6 +35,7 @@
 /**                                                        **/
 /**   AUTHOR     : Francois PELLEGRINI                     **/
 /**                Luca SCARANO (v3.1)                     **/
+/**                Sebastien FOURESTIER (v6.0)             **/
 /**                                                        **/
 /**   FUNCTION   : This module bipartitions an active      **/
 /**                graph using a multi-level scheme.       **/
@@ -51,6 +52,8 @@
 /**                                 to     20 mar 2005     **/
 /**                # Version 5.1  : from : 28 sep 2008     **/
 /**                                 to     27 mar 2011     **/
+/**                # Version 6.0  : from : 09 mar 2011     **/
+/**                                 to     22 dec 2011     **/
 /**                                                        **/
 /************************************************************/
 
@@ -64,8 +67,9 @@
 #include "common.h"
 #include "parser.h"
 #include "graph.h"
-#include "graph_coarsen.h"
 #include "arch.h"
+#include "mapping.h"
+#include "graph_coarsen.h"
 #include "bgraph.h"
 #include "bgraph_bipart_ml.h"
 #include "bgraph_bipart_st.h"
@@ -94,8 +98,10 @@ Bgraph * restrict const               coargrafptr, /*+ Coarser graph to build
 GraphCoarsenMulti * restrict * const  coarmultptr, /*+ Pointer to multinode table to build +*/
 const BgraphBipartMlParam * const     paraptr)    /*+ Method parameters                    +*/
 {
+  Gnum                comploadtmp;                /* Increase of imbalance range for coarse graph */
+
   if (graphCoarsen (&finegrafptr->s, &coargrafptr->s, coarmultptr,
-                    paraptr->coarnbr, paraptr->coarrat, paraptr->coartype) != 0)
+                    paraptr->coarnbr, paraptr->coarrat, NULL, NULL, 0, NULL) != 0)
     return (1);                                   /* Return if coarsening failed */
 
   if (finegrafptr->veextax != NULL) {             /* Merge external gains for coarsened vertices */
@@ -133,8 +139,11 @@ const BgraphBipartMlParam * const     paraptr)    /*+ Method parameters
   coargrafptr->parttax    = NULL;                 /* Do not allocate partition data yet                           */
   coargrafptr->frontab    = finegrafptr->frontab; /* Use frontier array of finer graph as coarse frontier array   */
 
-  coargrafptr->compload0min  = finegrafptr->compload0min; /* Only set constant partition parameters as others will be set on uncoarsening */
-  coargrafptr->compload0max  = finegrafptr->compload0max;
+  comploadtmp = (finegrafptr->levlnum + 1) +
+                (Gnum) (((double) MIN ((finegrafptr->compload0max - finegrafptr->compload0avg),
+                                       (finegrafptr->compload0avg - finegrafptr->compload0min))) * 0.05);
+  coargrafptr->compload0min  = finegrafptr->compload0min - comploadtmp; /* Only set constant partition parameters as others will be set on uncoarsening */
+  coargrafptr->compload0max  = finegrafptr->compload0max + comploadtmp;
   coargrafptr->compload0avg  = finegrafptr->compload0avg;
   coargrafptr->commloadextn0 = finegrafptr->commloadextn0;
   coargrafptr->commgainextn0 = finegrafptr->commgainextn0;
