@@ -1,4 +1,4 @@
-/* Copyright 2007-2011 ENSEIRB, INRIA & CNRS
+/* Copyright 2007-2012 IPB, Universite de Bordeaux, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -46,6 +46,8 @@
 /**                                 to   : 15 may 2008     **/
 /**                # Version 5.1  : from : 23 jun 2008     **/
 /**                                 to   : 20 feb 2011     **/
+/**                # Version 6.0  : from : 11 sep 2012     **/
+/**                                 to   : 03 oct 2012     **/
 /**                                                        **/
 /************************************************************/
 
@@ -96,21 +98,29 @@ Dgraph * restrict const             coargrafptr)  /*+ Coarse graph to build
   vertrcvnbr = vertgstnbr - vertlocnbr;
   vertsndnbr = finegrafptr->procsndnbr;
 
-  if ((coarptr->coarprivptr = memAllocGroup ((void **) (void *) /* Allocate distributed coarse graph private data */
-                                             &coargrafptr->procdsptab, (size_t) ((procglbnbr + 1) * sizeof (Gnum)),
-                                             &coargrafptr->proccnttab, (size_t) (procglbnbr       * sizeof (Gnum)),
-                                             &coargrafptr->procngbtab, (size_t) (procglbnbr       * sizeof (int)),
-                                             &coargrafptr->procrcvtab, (size_t) (procglbnbr       * sizeof (int)),
-                                             &coargrafptr->procsndtab, (size_t) (procglbnbr       * sizeof (int)), NULL)) == NULL) {
+  coarptr->coarprvptr = NULL;                     /* Assume nothing to free on error */
+  coarptr->multloctmp = NULL;
+  coarptr->nsndidxtab = NULL;
+  coarptr->nrcvidxtab = NULL;
+
+  if ((coarptr->coarprvptr = memAllocGroup ((void **) (void *) /* Allocate distributed coarse graph private data */
+                                            &coargrafptr->procdsptab, (size_t) ((procglbnbr + 1) * sizeof (Gnum)),
+                                            &coargrafptr->proccnttab, (size_t) (procglbnbr       * sizeof (Gnum)),
+                                            &coargrafptr->procngbtab, (size_t) (procglbnbr       * sizeof (int)),
+                                            &coargrafptr->procrcvtab, (size_t) (procglbnbr       * sizeof (int)),
+                                            &coargrafptr->procsndtab, (size_t) (procglbnbr       * sizeof (int)), NULL)) == NULL) {
     errorPrint ("dgraphCoarsenInit: out of memory (1)");
     return     (1);
   }
   coargrafptr->procvrttab = coargrafptr->procdsptab; /* Coarse graph has no holes */
 
-  if ((coarptr->multloctab = memAlloc (vertlocnbr * sizeof (DgraphCoarsenMulti))) == NULL) {
-    errorPrint ("dgraphCoarsenInit: out of memory (2)");
-    memFree    (coarptr->coarprivptr);
-    return     (1);
+  if (coarptr->multloctab == NULL) {              /* If no multinode array provided */
+    if ((coarptr->multloctab = memAlloc (vertlocnbr * sizeof (DgraphCoarsenMulti))) == NULL) {
+      errorPrint        ("dgraphCoarsenInit: out of memory (2)");
+      dgraphCoarsenExit (coarptr);
+      return            (1);
+    }
+    coarptr->multloctmp = coarptr->multloctab;    /* Array will have to be freed on error */
   }
 
   if (memAllocGroup ((void **) (void *)           /* Data used up to edge exchange phase at coarse graph build time */
@@ -119,10 +129,9 @@ Dgraph * restrict const             coargrafptr)  /*+ Coarse graph to build
                      &coarptr->coargsttax, (size_t) (vertgstnbr * sizeof (Gnum)),
                      &coarptr->procgsttax, (size_t) (vertrcvnbr * sizeof (int)), /* TRICK: Only purely ghost part of array will be used */
                      &coarptr->vrcvdattab, (size_t) (vertrcvnbr * sizeof (DgraphCoarsenVert)), NULL) == NULL) {
-    errorPrint ("dgraphCoarsenInit: out of memory (3)");
-    memFree    (coarptr->multloctab);
-    memFree    (coarptr->coarprivptr);
-    return     (1);
+    errorPrint        ("dgraphCoarsenInit: out of memory (3)");
+    dgraphCoarsenExit (coarptr);
+    return            (1);
   }
 
   buffsiz = 2 * MAX ((procngbnbr * sizeof (MPI_Request)), (procglbnbr * sizeof (int)));
@@ -133,11 +142,9 @@ Dgraph * restrict const             coargrafptr)  /*+ Coarse graph to build
                      &coarptr->dcntloctab, (size_t) (procglbnbr * sizeof (DgraphCoarsenCount)),
                      &coarptr->dcntglbtab, (size_t) (procglbnbr * sizeof (DgraphCoarsenCount)),
                      &coarptr->vsnddattab, (size_t) (vertsndnbr * sizeof (DgraphCoarsenVert)), NULL) == NULL) {
-    errorPrint ("dgraphCoarsenInit: out of memory (4)");
-    memFree    (coarptr->nrcvidxtab);
-    memFree    (coarptr->multloctab);
-    memFree    (coarptr->coarprivptr);
-    return     (1);
+    errorPrint        ("dgraphCoarsenInit: out of memory (4)");
+    dgraphCoarsenExit (coarptr);
+    return            (1);
   }
   coarptr->nrcvreqtab = (MPI_Request *) (void *) bufftab; /* TRICK: point-to-point requests and collective arrays share same space */
   coarptr->nsndreqtab = coarptr->nrcvreqtab + procngbnbr;
@@ -183,11 +190,12 @@ DgraphCoarsenData * restrict const    coarptr)    /*+ Coarsening data structure
 {
   if (coarptr->nsndidxtab != NULL)                /* Auxiliary array is released after first phase of coarse graph building */
     memFree (coarptr->nsndidxtab);
-  memFree (coarptr->nrcvidxtab);
-  if (coarptr->multloctab != NULL)                /* If multinode array not passed back to calling routine */
-    memFree (coarptr->multloctab);
-  if (coarptr->coarprivptr != NULL)               /* If ownership of coarse graph private data not yet transferred to it */
-    memFree (coarptr->coarprivptr);
+  if (coarptr->nrcvidxtab != NULL)
+    memFree (coarptr->nrcvidxtab);
+  if (coarptr->multloctmp != NULL)                /* If multinode array not provided nor passed back to calling routine */
+    memFree (coarptr->multloctmp);
+  if (coarptr->coarprvptr != NULL)                /* If ownership of coarse graph private data not yet transferred to it */
+    memFree (coarptr->coarprvptr);
 }
 
 static
@@ -199,6 +207,7 @@ DgraphCoarsenData * restrict const  coarptr)
   int                           procngbnbr;
   int                           procngbnum;
 
+  MPI_Comm                      proccomm   = coarptr->finegrafptr->proccomm;
   Dgraph * restrict const       grafptr    = coarptr->finegrafptr;
   const int * restrict const    procngbtab = grafptr->procngbtab;
   Gnum * restrict const         coargsttax = coarptr->coargsttax;
@@ -223,12 +232,12 @@ DgraphCoarsenData * restrict const  coarptr)
     vsnddsptab[procglbnum] = 2 * coarptr->vsnddsptab[procglbnum];
   }
 
-  if (MPI_Alltoall (vsndcnttab, 1, MPI_INT, coarptr->vrcvcnttab, 1, MPI_INT, grafptr->proccomm) != MPI_SUCCESS) {
+  if (MPI_Alltoall (vsndcnttab, 1, MPI_INT, coarptr->vrcvcnttab, 1, MPI_INT, proccomm) != MPI_SUCCESS) {
     errorPrint ("dgraphCoarsenBuildColl: communication error (1)");
     return     (1);
   }
   if (MPI_Alltoallv (coarptr->vsnddattab, vsndcnttab,          vsnddsptab, GNUM_MPI,
-                     coarptr->vrcvdattab, coarptr->vrcvcnttab, vrcvdsptab, GNUM_MPI, grafptr->proccomm) != MPI_SUCCESS) {
+                     coarptr->vrcvdattab, coarptr->vrcvcnttab, vrcvdsptab, GNUM_MPI, proccomm) != MPI_SUCCESS) {
     errorPrint ("dgraphCoarsenBuildColl: communication error (2)");
     return     (1);
   }
@@ -277,6 +286,7 @@ DgraphCoarsenData * restrict const  coarptr)
   int                           procngbnum;
   int                           vrcvreqnbr;
 
+  MPI_Comm                      proccomm   = coarptr->finegrafptr->proccomm;
   Dgraph * restrict const       grafptr    = coarptr->finegrafptr;
   const int * restrict const    procngbtab = grafptr->procngbtab;
   Gnum * restrict const         coargsttax = coarptr->coargsttax;
@@ -296,7 +306,7 @@ DgraphCoarsenData * restrict const  coarptr)
       procngbnum = (procngbnum + (procngbnbr - 1)) % procngbnbr; /* Pre-decrement neighbor rank */
       procglbnum = procngbtab[procngbnum];
       if (MPI_Irecv (coarptr->vrcvdattab + vrcvdsptab[procglbnum], 2 * (vrcvdsptab[procglbnum + 1] - vrcvdsptab[procglbnum]), GNUM_MPI,
-                     procglbnum, TAGCOARSEN, grafptr->proccomm, &coarptr->nrcvreqtab[procngbnum]) != MPI_SUCCESS) {
+                     procglbnum, TAGCOARSEN, proccomm, &coarptr->nrcvreqtab[procngbnum]) != MPI_SUCCESS) {
         errorPrint ("dgraphCoarsenBuildPtop: communication error (1)");
         return     (1);
       }
@@ -308,7 +318,7 @@ DgraphCoarsenData * restrict const  coarptr)
 
       procglbnum = procngbtab[procngbnum];
       if (MPI_Isend (coarptr->vsnddattab + vsnddsptab[procglbnum], 2 * (nsndidxtab[procngbnum] - vsnddsptab[procglbnum]), GNUM_MPI,
-                     procglbnum, TAGCOARSEN, grafptr->proccomm, &coarptr->nsndreqtab[procngbnum]) != MPI_SUCCESS) {
+                     procglbnum, TAGCOARSEN, proccomm, &coarptr->nsndreqtab[procngbnum]) != MPI_SUCCESS) {
         errorPrint ("dgraphCoarsenBuildPtop: communication error (2)");
         return     (1);
       }
@@ -430,6 +440,7 @@ DgraphCoarsenData * restrict const  coarptr)
   int * restrict                ercvdbgtab;
 #endif /* SCOTCH_DEBUG_DGRAPH2 */
 
+  MPI_Comm                                  proccomm    = coarptr->finegrafptr->proccomm;
   Dgraph * restrict const                   grafptr     = coarptr->finegrafptr;
   Dgraph * restrict const                   coargrafptr = coarptr->coargrafptr;
   Gnum * restrict const                     coargsttax  = coarptr->coargsttax;
@@ -519,13 +530,16 @@ DgraphCoarsenData * restrict const  coarptr)
     }
   }
 
-  coarptr->multloctab = memRealloc (coarptr->multloctab, coarptr->multlocnbr * sizeof (DgraphCoarsenMulti)); /* In the mean time, resize multinode array */
+  if (coarptr->multloctmp != NULL) {              /* If we allocated the multinode array */
+    coarptr->multloctmp =
+    coarptr->multloctab = memRealloc (coarptr->multloctab, coarptr->multlocnbr * sizeof (DgraphCoarsenMulti)); /* In the mean time, resize multinode array */
+  }
 
   if ((((grafptr->flagval & DGRAPHCOMMPTOP) != 0) ? dgraphCoarsenBuildPtop : dgraphCoarsenBuildColl) (coarptr) != 0)
     return (1);
 
 #ifdef SCOTCH_DEBUG_DGRAPH2
-  if (MPI_Barrier (grafptr->proccomm) != MPI_SUCCESS) {
+  if (MPI_Barrier (proccomm) != MPI_SUCCESS) {
     errorPrint ("dgraphCoarsenBuild: communication error (1)");
     return     (1);
   }
@@ -581,7 +595,7 @@ DgraphCoarsenData * restrict const  coarptr)
 
   cheklocval = 0;
   coargrafptr->flagval = DGRAPHFREETABS | DGRAPHFREEPRIV | DGRAPHVERTGROUP; /* Coarse graph is not yet based */
-  coarptr->coarprivptr = NULL;                    /* Transfer ownership of private arrays to coarse graph    */
+  coarptr->coarprvptr = NULL;                     /* Transfer ownership of private arrays to coarse graph    */
   if (memAllocGroup ((void **) (void *)
                      &coargrafptr->vertloctax, (size_t) ((coarptr->multlocnbr + 1) * sizeof (Gnum)),
                      &coargrafptr->veloloctax, (size_t) ( coarptr->multlocnbr      * sizeof (Gnum)), NULL) == NULL) {
@@ -609,7 +623,7 @@ DgraphCoarsenData * restrict const  coarptr)
     cheklocval = 1;
   }
 #ifdef SCOTCH_DEBUG_DGRAPH1                       /* Communication cannot be overlapped by a useful one */
-  if (MPI_Allreduce (&cheklocval, &chekglbval, 1, MPI_INT, MPI_SUM, grafptr->proccomm) != MPI_SUCCESS) {
+  if (MPI_Allreduce (&cheklocval, &chekglbval, 1, MPI_INT, MPI_SUM, proccomm) != MPI_SUCCESS) {
     errorPrint ("dgraphCoarsenBuild: communication error (2)");
     chekglbval = 1;
   }
@@ -617,7 +631,7 @@ DgraphCoarsenData * restrict const  coarptr)
   chekglbval = cheklocval;
 #endif /* SCOTCH_DEBUG_DGRAPH1 */
   if (chekglbval != 0) {
-    dgraphExit (coargrafptr);
+    dgraphFree (coargrafptr);
     return     (1);
   }
 
@@ -630,9 +644,6 @@ DgraphCoarsenData * restrict const  coarptr)
   coargrafptr->veloloctax -= coargrafptr->baseval;
   coargrafptr->edgeloctax -= coargrafptr->baseval;
   coargrafptr->edloloctax -= coargrafptr->baseval;
-  coargrafptr->proccomm    = grafptr->proccomm;
-  coargrafptr->procglbnbr  = grafptr->procglbnbr;
-  coargrafptr->proclocnum  = grafptr->proclocnum;
 
   for (procngbnum = procnum = 0, esnddspval = 0; procngbnum < procngbnbr; procngbnum ++) {
     int                 procglbnum;
@@ -697,7 +708,7 @@ DgraphCoarsenData * restrict const  coarptr)
   }
 
 #ifdef SCOTCH_DEBUG_DGRAPH2
-  if (MPI_Alltoall (esndcnttab, 1, MPI_INT, ercvdbgtab, 1, MPI_INT, grafptr->proccomm) != MPI_SUCCESS) {
+  if (MPI_Alltoall (esndcnttab, 1, MPI_INT, ercvdbgtab, 1, MPI_INT, proccomm) != MPI_SUCCESS) {
     errorPrint ("dgraphCoarsenBuild: communication error (3)");
     return     (1);
   }
@@ -709,7 +720,7 @@ DgraphCoarsenData * restrict const  coarptr)
   }
 #endif /* SCOTCH_DEBUG_DGRAPH2 */
   if (MPI_Alltoallv (esnddattab, esndcnttab, esnddsptab, GNUM_MPI,
-                     ercvdattab, ercvcnttab, ercvdsptab, GNUM_MPI, grafptr->proccomm) != MPI_SUCCESS) {
+                     ercvdattab, ercvcnttab, ercvdsptab, GNUM_MPI, proccomm) != MPI_SUCCESS) {
     errorPrint ("dgraphCoarsenBuild: communication error (4)");
     return     (1);
   }
@@ -856,7 +867,7 @@ DgraphCoarsenData * restrict const  coarptr)
   reduloctab[2] = coardegrlocmax;                 /* Get local maximum degree */
   reduloctab[3] = coargrafptr->edgelocnbr;
 
-  if (dgraphAllreduceMaxSum (reduloctab, reduglbtab, 3, 1, coargrafptr->proccomm) != 0) {
+  if (dgraphAllreduceMaxSum (reduloctab, reduglbtab, 3, 1, proccomm) != 0) {
     errorPrint ("dgraphCoarsenBuild: communication error (5)");
     return     (1);
   }
@@ -890,9 +901,10 @@ int
 dgraphCoarsen (
 Dgraph * restrict const               finegrafptr, /*+ Graph to coarsen                    +*/
 Dgraph * restrict const               coargrafptr, /*+ Coarse graph to build               +*/
-DgraphCoarsenMulti * restrict * const multlocptr, /*+ Pointer to based multinode table     +*/
+DgraphCoarsenMulti * restrict * const multlocptr, /*+ Pointer to un-based multinode array  +*/
 const Gnum                            passnbr,    /*+ Number of coarsening passes to go    +*/
 const Gnum                            coarnbr,    /*+ Minimum number of coarse vertices    +*/
+const int                             flagval,    /*+ Flag value                           +*/
 const int                             foldval,    /*+ Allow fold/dup or fold or no fold    +*/
 const Gnum                            dupmax,     /*+ Minimum number of vertices to do dup +*/
 const double                          coarrat)    /*+ Maximum contraction ratio            +*/
@@ -910,9 +922,6 @@ const double                          coarrat)    /*+ Maximum contraction ratio
   int                       procnum;
   int                       o;
 
-  memSet (coargrafptr, 0, sizeof (Dgraph));       /* Preset coarse graph data                      */
-  *multlocptr = NULL;                             /* Assume we will not create any multinode array */
-
 #ifdef SCOTCH_DEBUG_DGRAPH1
   if (coarrat < 0.5L)                             /* If impossible coarsening ratio wanted */
     return (1);                                   /* We will never succeed                 */
@@ -927,6 +936,8 @@ const double                          coarrat)    /*+ Maximum contraction ratio
     return     (2);
   }
 
+  matedat.c.flagval    = flagval;
+  matedat.c.multloctab = *multlocptr;             /* Propagate the provided multinode array or NULL if it has to be allocated */
   cheklocval  = dgraphCoarsenInit (&matedat.c, finegrafptr, coargrafptr);
   cheklocval |= dgraphMatchInit   (&matedat, 0.5F);
 
@@ -942,7 +953,8 @@ const double                          coarrat)    /*+ Maximum contraction ratio
     return (2);
 
   for (passnum = 0; passnum < passnbr; passnum ++) {
-    dgraphMatchHy (&matedat);
+    ((passnum == 0) ? dgraphMatchHl : dgraphMatchHy) (&matedat); /* If first pass, process lightest vertices first */
+
     if ((((finegrafptr->flagval & DGRAPHCOMMPTOP) != 0) ? dgraphMatchSyncPtop : dgraphMatchSyncColl) (&matedat) != 0) {
       errorPrint        ("dgraphCoarsen: cannot perform matching");
       dgraphMatchExit   (&matedat);
@@ -1015,11 +1027,11 @@ const double                          coarrat)    /*+ Maximum contraction ratio
 
   o = 0;                                          /* Assume everything is now all right */
   multloctax = matedat.c.multloctab - finegrafptr->baseval;
-  matedat.c.multloctab = NULL;                    /* So that it will not be freed */
-  dgraphCoarsenExit (&matedat.c);
+  matedat.c.multloctmp = NULL;                    /* So that it will not be freed    */
+  dgraphCoarsenExit (&matedat.c);                 /* Free all other temporary arrays */
 
 #ifdef PTSCOTCH_FOLD_DUP
-  if ((coargrafptr->procglbnbr >= 2) &&
+  if ((coargrafptr->procglbnbr >= 2) &&           /* Never do folding in case of user-provided multinode array! */
       ((foldval != 0) || (dupmax > (coargrafptr->vertglbnbr / coargrafptr->procglbnbr)))) {
     Dgraph                coargrafdat;            /* Coarse graph data before folding */
     MPI_Datatype          coarmultype;
@@ -1046,10 +1058,11 @@ const double                          coarrat)    /*+ Maximum contraction ratio
     dgraphExit    (&coargrafdat);                 /* Free unfolded graph */
     MPI_Type_free (&coarmultype);
     memFree (coarmultptr + finegrafptr->baseval); /* TRICK: use preserved baseval */
+    *multlocptr = multloctax + finegrafptr->baseval; /* Return un-based pointer   */
   }
+  else
 #endif /* PTSCOTCH_FOLD_DUP */
-
-  *multlocptr = multloctax;
+    *multlocptr = matedat.c.multloctab;           /* Return un-based pointer */
 
   return (o);
 }
