@@ -1,4 +1,4 @@
-/* Copyright 2004,2007-2009 ENSEIRB, INRIA & CNRS
+/* Copyright 2004,2007-2009,2011 ENSEIRB, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -34,6 +34,7 @@
 /**   NAME       : graph_induce.c                          **/
 /**                                                        **/
 /**   AUTHOR     : Francois PELLEGRINI                     **/
+/**                Sebastien FOURESTIER (v6.0)             **/
 /**                                                        **/
 /**   FUNCTION   : This module handles the source graph    **/
 /**                subgraph-making functions.              **/
@@ -56,6 +57,8 @@
 /**                                 to     11 jun 2008     **/
 /**                # Version 5.1  : from : 01 jan 2009     **/
 /**                                 to     01 jan 2009     **/
+/**                # Version 6.0  : from : 29 mar 2011     **/
+/**                                 to     29 mar 2011     **/
 /**                                                        **/
 /**   NOTES      : # Several algorithms, such as the       **/
 /**                  active graph building routine of      **/
@@ -91,6 +94,10 @@
 ** array if the original graph does not have
 ** a vnumtab, or the proper subset of the
 ** original vnumtab else.
+** if rvnutax (reverse vnumtax) is not NULL
+** (and of size vertnbr), it will be filled
+** by new vertex numbers of ~0 if the vertex
+** is not in the induced graph.
 ** It returns:
 ** - 0   : on success.
 ** - !0  : on error.
@@ -100,7 +107,8 @@ int
 graphInduceList (
 const Graph * restrict const    orggrafptr,
 const VertList * restrict const indlistptr,
-Graph * restrict const          indgrafptr)
+Graph * restrict const          indgrafptr,
+Gnum * restrict const           rvnutax)
 {
   Gnum * restrict     orgindxtax;                 /* Based access to vertex translation array       */
   Gnum                indvertnbr;                 /* Number of vertices in induced graph            */
@@ -143,12 +151,23 @@ Graph * restrict const          indgrafptr)
   if (orggrafptr->edlotax != NULL)                /* If graph has edge weights */
     indedgenbr *= 2;                              /* Account for edge weights  */
 
-  if (memAllocGroup ((void *)
-        &indedgetab, (size_t) (indedgenbr          * sizeof (Gnum)), /* Pre-allocate space for edgetab (and edlotab)          */
-        &orgindxtax, (size_t) (orggrafptr->vertnbr * sizeof (Gnum)), NULL) == NULL) { /* orgindxtab is at the end of the heap */
-    errorPrint ("graphInduceList: out of memory (3)");
-    graphExit  (indgrafptr);
-    return     (1);
+  if (rvnutax == NULL) {
+    if (memAllocGroup ((void *)
+          &indedgetab, (size_t) (indedgenbr          * sizeof (Gnum)), /* Pre-allocate space for edgetab (and edlotab)          */
+          &orgindxtax, (size_t) (orggrafptr->vertnbr * sizeof (Gnum)), NULL) == NULL) { /* orgindxtab is at the end of the heap */
+      errorPrint ("graphInduceList: out of memory (3)");
+      graphExit  (indgrafptr);
+      return     (1);
+    }
+    orgindxtax -= orggrafptr->baseval;
+  }
+  else {
+    if ((indedgetab = memAlloc (indedgenbr * sizeof (Gnum))) == NULL) { /* Pre-allocate space for edgetab (and edlotab) */
+      errorPrint ("graphInduceList: out of memory (4)");
+      graphExit  (indgrafptr);
+      return     (1);
+    }
+    orgindxtax = rvnutax;
   }
 
 /* TODO: Vertex list can be kept as it is the one of *graphOrderNd */
@@ -156,8 +175,7 @@ Graph * restrict const          indgrafptr)
   memCpy (indgrafptr->vnumtax + indgrafptr->baseval, /* Copy vertex number array from list */
           indlistptr->vnumtab, indvertnbr * sizeof (Gnum));
 
-  memSet (orgindxtax, ~0, orggrafptr->vertnbr * sizeof (Gnum)); /* Preset index array */
-  orgindxtax -= orggrafptr->baseval;
+  memSet (orgindxtax + orggrafptr->baseval, ~0, orggrafptr->vertnbr * sizeof (Gnum)); /* Preset index array */
 
   for (indvertnum = indgrafptr->baseval, indedgenbr = 0; /* Fill index array */
        indvertnum < indgrafptr->baseval + indvertnbr; indvertnum ++) {
@@ -179,6 +197,10 @@ Graph * restrict const          indgrafptr)
 ** selected vertices if the original graph
 ** does not have a vnumtab, or the proper
 ** subset of the original vnumtab else.
+** if rvnutax (reverse vnumtax) is not NULL
+** (and of size vertnbr), it will be filled
+** by new vertex numbers of ~0 if the vertex
+** is not in the induced graph.
 ** It returns:
 ** - 0   : on success.
 ** - !0  : on error.
@@ -190,9 +212,9 @@ const Graph * restrict const  orggrafptr,         /* Pointer to original graph
 const GraphPart * const       orgparttax,         /* Based array of vertex partition flags */
 const Gnum                    indvertnbr,         /* Number of vertices in selected part   */
 const GraphPart               indpartval,         /* Partition value of vertices to keep   */
-Graph * restrict const        indgrafptr)         /* Pointer to induced subgraph           */
+Graph * restrict const        indgrafptr,         /* Pointer to induced subgraph           */
+Gnum * restrict const         rvnutax)
 {
-  Gnum * restrict             orgindxtab;         /* Original to induced vertex translation array   */
   Gnum * restrict             orgindxtax;         /* Based access to vertex translation array       */
   Gnum                        indvertnum;         /* Number of current vertex in induced graph      */
   Gnum * restrict             indedgetab;         /* Pointer to pre-allocated edge array            */
@@ -231,14 +253,24 @@ Graph * restrict const        indgrafptr)         /* Pointer to induced subgraph
   indgrafptr->vertnbr  = indvertnbr;
   indgrafptr->vertnnd  = indvertnbr + indgrafptr->baseval;
 
-  if (memAllocGroup ((void *)
-        &indedgetab, (size_t) (indedgenbr          * sizeof (Gnum)), /* Pre-allocate space for edgetab (and edlotab)          */
-        &orgindxtab, (size_t) (orggrafptr->vertnbr * sizeof (Gnum)), NULL) == NULL) { /* orgindxtab is at the end of the heap */
-    errorPrint ("graphInducePart: out of memory (3)");
-    graphExit  (indgrafptr);
-    return     (1);
+  if (rvnutax == NULL) {
+    if (memAllocGroup ((void *)
+          &indedgetab, (size_t) (indedgenbr          * sizeof (Gnum)), /* Pre-allocate space for edgetab (and edlotab)          */
+          &orgindxtax, (size_t) (orggrafptr->vertnbr * sizeof (Gnum)), NULL) == NULL) { /* orgindxtab is at the end of the heap */
+      errorPrint ("graphInducePart: out of memory (3)");
+      graphExit  (indgrafptr);
+      return     (1);
+    }
+    orgindxtax -= orggrafptr->baseval;
+  }
+  else {
+    if ((indedgetab = memAlloc (indedgenbr * sizeof (Gnum))) == NULL) { /* Pre-allocate space for edgetab (and edlotab) */
+      errorPrint ("graphInduceList: out of memory (4)");
+      graphExit  (indgrafptr);
+      return     (1);
+    }
+    orgindxtax = rvnutax;
   }
-  orgindxtax = orgindxtab - orggrafptr->baseval;
 
   for (indvertnum = indgrafptr->baseval, indedgenbr = 0, orgvertnum = orggrafptr->baseval; /* Fill index array */
        orgvertnum < orggrafptr->vertnnd; orgvertnum ++) {
