@@ -34,6 +34,7 @@
 /**   NAME       : kgraph_map_rb_map.c                     **/
 /**                                                        **/
 /**   AUTHOR     : Francois PELLEGRINI                     **/
+/**                Sebastien FOURESTIER (v6.0)             **/
 /**                                                        **/
 /**   FUNCTION   : This module performs the Dual Recursive **/
 /**                Bipartitioning mapping algorithm.       **/
@@ -63,7 +64,9 @@
 /**                # Version 4.0  : from : 12 jan 2004     **/
 /**                                 to     06 mar 2005     **/
 /**                # Version 5.1  : from : 22 nov 2007     **/
-/**                                 to     31 aug 2011     **/
+/**                                 to     04 feb 2009     **/
+/**                # Version 6.0  : from : 03 mar 2011     **/
+/**                                 to     05 sep 2011     **/
 /**                                                        **/
 /**   NOTES      : # This code is a complete rewrite of    **/
 /**                  the original code of kgraphMapRb(),   **/
@@ -116,15 +119,15 @@ const KgraphMapRbParam * restrict const paraptr)
 {
   int                 flagval;
 
-  memSet (grafptr->m.parttax + grafptr->m.baseval, 0, grafptr->s.vertnbr * sizeof (ArchDomNum)); /* Initialize partition data */
+  memSet (grafptr->m.parttax + grafptr->s.baseval, 0, grafptr->s.vertnbr * sizeof (ArchDomNum)); /* Initialize partition data */
   grafptr->m.domnnbr    = 1;                      /* Only one valid domain to date */
   grafptr->m.domntab[0] = grafptr->m.domnorg;     /* All vertices are mapped to it */
 
   flagval = 0;
   poolptr->grafptr = NULL;                        /* Assume we don't need top level graph data */
-  if (archVar (&grafptr->m.archdat) != 0)
+  if (archVar (grafptr->m.archptr) != 0)
     flagval |= KGRAPHMAPRBMAPARCHVAR;
-  if (archPart (&grafptr->m.archdat) != 0)
+  if (archPart (grafptr->m.archptr) != 0)
     flagval |= KGRAPHMAPRBMAPARCHCMPLT;
   else
     poolptr->grafptr = &grafptr->s;               /* We will need top-level graph data */
@@ -177,8 +180,11 @@ KgraphMapRbMapPoolData * restrict const poolptr)
   int                 jobnum;
 
   for (jobnum = 0; jobnum < poolptr->mappptr->domnnbr; jobnum ++) { /* For all potential jobs in both pools */
-    if (poolptr->jobtab[jobnum].poolflag != 0) /* If job slot is active                                     */
+    if (poolptr->jobtab[jobnum].poolflag != 0) { /* If job slot is active                                   */
       graphFree (&poolptr->jobtab[jobnum].grafdat); /* Free job graph, if not clone of original graph       */
+      if (poolptr->jobtab[jobnum].rvnutax != NULL)
+        memFree (poolptr->jobtab[jobnum].rvnutax);
+    }
   }
 
   if (poolptr->mappptr->domntab != poolptr->domntmp) { /* If current mapping domain array is not original domain array      */
@@ -453,6 +459,9 @@ KgraphMapRbMapJob * const       jobnewptr1)
   KgraphMapRbMapJob * restrict  jobnewtab[2];
   int                           i;
 
+  if (joboldptr->rvnutax != NULL)
+    memFree (joboldptr->rvnutax + joboldptr->grafdat.baseval);
+
   jobnewtab[0] = jobnewptr0;
   jobnewtab[1] = jobnewptr1;
 
@@ -681,7 +690,9 @@ const Anum                              jobsubnum1)
 int
 kgraphMapRbMap (
 Kgraph * restrict const                 grafptr,
-const KgraphMapRbParam * restrict const paraptr)
+const KgraphMapRbParam * restrict const paraptr,
+const Graph * restrict const            fixgrafptr,
+Gnum * const                            fixrvnutax)
 {
   KgraphMapRbMapPoolData  pooldat;                /* Data for handling jobs and job pools */
   ArchDom                 domsubtab[2];           /* Subdomains of current job domain     */
@@ -700,7 +711,7 @@ const KgraphMapRbParam * restrict const paraptr)
   if (kgraphMapRbMapPoolInit (&pooldat, grafptr, paraptr) != 0) /* Initialize pool data; done first for kgraphMapRbMapPoolExit() to succeed afterwards */
     return (1);
 
-  if ((((pooldat.flagval & KGRAPHMAPRBMAPARCHVAR) == 0) && (archDomSize (&grafptr->m.archdat, &grafptr->m.domnorg) <= 1)) || /* If single-vertex domain */
+  if ((((pooldat.flagval & KGRAPHMAPRBMAPARCHVAR) == 0) && (archDomSize (grafptr->m.archptr, &grafptr->m.domnorg) <= 1)) || /* If single-vertex domain */
       (((pooldat.flagval & KGRAPHMAPRBMAPARCHVAR) != 0) && (grafptr->s.vertnbr <= 1))) { /* Or if variable-sized architecture with single vertex graph  */
     kgraphMapRbMapPoolExit (&pooldat);
     return (0);                                   /* Job already done */
@@ -709,6 +720,7 @@ const KgraphMapRbParam * restrict const paraptr)
   pooldat.jobtab[0].domnorg = grafptr->m.domnorg; /* Build first job                         */
   pooldat.jobtab[0].grafdat = grafptr->s;         /* Clone original graph as first job graph */
   pooldat.jobtab[0].grafdat.flagval &= ~GRAPHFREETABS; /* Do not free its arrays on exit     */
+  pooldat.jobtab[0].rvnutax = NULL;               /* No reverse vnumtax for original graph   */
   kgraphMapRbMapPoolFrst (&pooldat, &pooldat.jobtab[0]); /* Add initial job                  */
 
   comploadmin = (1.0 - paraptr->kbalval) * grafptr->comploadrat; /* Ratio can have been tilted when working on subgraph */
@@ -723,14 +735,15 @@ const KgraphMapRbParam * restrict const paraptr)
       jobsubnum[0] = joborgptr - pooldat.jobtab;  /* Get current (and first son) job slot number before possible move of pointers */
       joborgdat = *joborgptr;                     /* Save current job data (clone graph)                                          */
 
-      if (archDomBipart (&grafptr->m.archdat, &joborgdat.domnorg, &domsubtab[0], &domsubtab[1]) != 0) {
+      if (archDomBipart (grafptr->m.archptr, &joborgdat.domnorg, &domsubtab[0], &domsubtab[1]) != 0) {
         errorPrint ("kgraphMapRbMap: cannot bipartition domain");
         kgraphMapRbMapPoolExit (&pooldat);        /* Copied graph will be freed as not yet removed */
         return (1);
       }
 
-      if (bgraphInit (&bipgrafdat, &joborgdat.grafdat, /* Create bipartition graph */
-                      pooldat.grafptr, pooldat.mappptr, domsubtab) != 0) {
+      if (bgraphInit (&bipgrafdat, &joborgdat.grafdat, pooldat.grafptr, &grafptr->r.m, fixgrafptr, /* Create bipartition graph */
+                      fixrvnutax, joborgdat.rvnutax, grafptr->r.cmloval, grafptr->r.vmlotax,
+                      grafptr->pfixtax, pooldat.mappptr, domsubtab) != 0) {
         errorPrint ("kgraphMapRbMap: cannot create bipartition graph");
         kgraphMapRbMapPoolExit (&pooldat);        /* Copied graph will be freed as not yet removed */
         return (1);
@@ -741,7 +754,7 @@ const KgraphMapRbParam * restrict const paraptr)
       if ((pooldat.flagval & KGRAPHMAPRBMAPARCHVAR) == 0) { /* If not variable-sized, impose constraints on bipartition */
         double              comploadavg;
 
-        comploadavg = (double) bipgrafdat.s.velosum / (double) archDomWght (&grafptr->m.archdat, &joborgdat.domnorg);
+        comploadavg = (double) bipgrafdat.s.velosum / (double) archDomWght (grafptr->m.archptr, &joborgdat.domnorg);
         bipgrafdat.compload0min = bipgrafdat.compload0avg -
                                   (Gnum) MIN ((comploadmax - comploadavg) * (double) bipgrafdat.domwght[0],
                                               (comploadavg - comploadmin) * (double) bipgrafdat.domwght[1]);
@@ -762,7 +775,7 @@ const KgraphMapRbParam * restrict const paraptr)
       if ((partval = 1, bipgrafdat.compload0 == 0) || /* If no bipartition found */
           (partval = 0, bipgrafdat.compload0 == bipgrafdat.s.velosum)) {
         if (((pooldat.flagval & KGRAPHMAPRBMAPARCHVAR) != 0) || /* If architecture is variable-sized     */
-            (archDomSize (&grafptr->m.archdat, &domsubtab[partval]) <= 1)) { /* Or domain is terminal    */
+            (archDomSize (grafptr->m.archptr, &domsubtab[partval]) <= 1)) { /* Or domain is terminal    */
           pooldat.domntab[jobsubnum[0]] = joborgdat.domnorg; /* Update domain in next pool               */
           kgraphMapRbMapPoolRemv (&pooldat, &joborgdat); /* Remove job from pool as long as graph exists */
           bgraphExit (&bipgrafdat);               /* Free bipartitioning data as well as current graph   */
@@ -805,8 +818,9 @@ const KgraphMapRbParam * restrict const paraptr)
         jobsubptr->poollink.next = NULL;
         jobsubptr->prioval =                      /* Prevent Valgrind from yelling in kgraphMapRbMapPoolRemv()/Updt1()/Updt2() */
         jobsubptr->priolvl = 0;
+        jobsubptr->rvnutax = NULL;
 
-        if ((((pooldat.flagval & KGRAPHMAPRBMAPARCHVAR) == 0) && (archDomSize (&grafptr->m.archdat, &domsubtab[i]) <= 1)) || /* If single-vertex domain */
+        if ((((pooldat.flagval & KGRAPHMAPRBMAPARCHVAR) == 0) && (archDomSize (grafptr->m.archptr, &domsubtab[i]) <= 1)) || /* If single-vertex domain */
             (((pooldat.flagval & KGRAPHMAPRBMAPARCHVAR) != 0) && (jobsubsiz[i] <= 1))) { /* Or if variable-sized architecture with single vertex graph  */
           jobsubsiz[i] = 0;                       /* Cancel subjob */
           continue;
@@ -814,7 +828,15 @@ const KgraphMapRbParam * restrict const paraptr)
 
         partval = i;                              /* At least this subjob works   */
 
-        if (graphInducePart (&bipgrafdat.s, bipgrafdat.parttax, jobsubsiz[i], (GraphPart) i, &jobsubptr->grafdat) != 0) {
+        if (fixrvnutax != NULL) {
+          if ((jobsubptr->rvnutax = memAlloc (grafptr->s.vertnbr * sizeof (Gnum))) == NULL) {
+            errorPrint ("kgraphMapRbMap: out of memory");
+            return     (1);
+          }
+          memSet (jobsubptr->rvnutax, ~0, grafptr->s.vertnbr * sizeof (Gnum));
+          jobsubptr->rvnutax -= grafptr->s.baseval;
+        }
+        if (graphInducePart (&bipgrafdat.s, bipgrafdat.parttax, jobsubsiz[i], (GraphPart) i, &jobsubptr->grafdat, jobsubptr->rvnutax) != 0) {
           errorPrint             ("kgraphMapRbMap: cannot create induced subgraph");
           bgraphExit             (&bipgrafdat);
           kgraphMapRbMapPoolExit (&pooldat);
@@ -824,8 +846,11 @@ const KgraphMapRbParam * restrict const paraptr)
         jobsubptr->domnorg  = domsubtab[i];
       }
 
-      if ((jobsubsiz[0] | jobsubsiz[1]) == 0)     /* If both subjobs do not need further processing */
+      if ((jobsubsiz[0] | jobsubsiz[1]) == 0) {   /* If both subjobs do not need further processing */
         kgraphMapRbMapPoolRemv (&pooldat, &joborgdat);
+        if (fixrvnutax != NULL)
+          memFree (joborgdat.rvnutax);
+      }
       else if (jobsubsiz[1 - partval] == 0)       /* If one of the subjobs only needs further processing */
         kgraphMapRbMapPoolUpdt1 (&pooldat, &joborgdat, bipgrafdat.parttax, &pooldat.jobtab[jobsubnum[partval]], (GraphPart) partval);
       else
