@@ -1,4 +1,4 @@
-/* Copyright 2004,2007,2008,2010 ENSEIRB, INRIA & CNRS
+/* Copyright 2004,2007,2008,2010,2012 IPB, Universite de Bordeaux, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -49,6 +49,8 @@
 /**                                 to     04 aug 2007     **/
 /**                # Version 5.1  : from : 30 oct 2007     **/
 /**                                 to     14 aug 2010     **/
+/**                # Version 6.0  : from : 08 jan 2012     **/
+/**                                 to     14 nov 2012     **/
 /**                                                        **/
 /************************************************************/
 
@@ -272,7 +274,7 @@ SCOTCH_Strat * const        stratptr)             /*+ Ordering strategy
   }
 
   if (*((Strat **) stratptr) == NULL)             /* Set default ordering strategy if necessary */
-    SCOTCH_stratGraphOrderBuild (stratptr, SCOTCH_STRATQUALITY, 0.2);
+    SCOTCH_stratGraphOrderBuild (stratptr, SCOTCH_STRATQUALITY, 0, 0.2);
 
   ordstratptr = *((Strat **) stratptr);
   if (ordstratptr->tabl != &hgraphorderststratab) {
@@ -354,6 +356,8 @@ SCOTCH_Strat * const        stratptr)             /*+ Ordering strategy
     halgrafptr = &halgraftmp;
   }
 
+  intRandInit ();                                 /* Check that random number generator is initialized */
+
   hgraphOrderSt (halgrafptr, &libordeptr->o, 0, cblkptr, ordstratptr);
 
   if (halgrafptr != &halgrafdat)                  /* If induced subgraph created */
@@ -485,16 +489,56 @@ const char * const          string)
 
 int
 SCOTCH_stratGraphOrderBuild (
-SCOTCH_Strat * const        stratptr,             /*+ Strategy to create      +*/
-const SCOTCH_Num            flagval,              /*+ Desired characteristics +*/
-const double                balrat)               /*+ Desired imbalance ratio +*/
+SCOTCH_Strat * const        stratptr,             /*+ Strategy to create                 +*/
+const SCOTCH_Num            flagval,              /*+ Desired characteristics            +*/
+const SCOTCH_Num            levlnbr,              /*+ Number of nested dissection levels +*/
+const double                balrat)               /*+ Desired imbalance ratio            +*/
 {
   char                bufftab[8192];              /* Should be enough */
+  char                levltab[32];
   char                bbaltab[32];
-
-  strcpy (bufftab, "c{rat=0.7,cpr=n{sep=/(vert>240)?m{type=h,rat=0.7,vert=100,low=h{pass=10},asc=b{width=3,bnd=f{bal=<BBAL>},org=(|h{pass=10})f{bal=<BBAL>}}}|m{type=h,rat=0.7,vert=100,low=h{pass=10},asc=b{width=3,bnd=f{bal=<BBAL>},org=(|h{pass=10})f{bal=<BBAL>}}};,ole=f{cmin=0,cmax=100000,frat=0.0},ose=g},unc=n{sep=/(vert>240)?m{type=h,rat=0.7,vert=100,low=h{pass=10},asc=b{width=3,bnd=f{bal=<BBAL>},org=(|h{pass=10})f{bal=<BBAL>}}}|m{type=h,rat=0.7,vert=100,low=h{pass=10},asc=b{width=3,bnd=f{bal=<BBAL>},org=(|h{pass=10})f{bal=<BBAL>}}};,ole=f{cmin=15,cmax=100000,frat=0.0},ose=g}}");
+  char *              sepaptr;
+  char *              tstsptr;
+  char *              oleaptr;
+  char *              osepptr;
 
   sprintf (bbaltab, "%lf", balrat);
+  sprintf (levltab, GNUMSTRING, levlnbr);
+
+  strcpy (bufftab, "c{rat=0.7,cpr=n{sep=/(<TSTS>)?m{rat=0.7,vert=100,low=h{pass=10},asc=b{width=3,bnd=f{bal=<BBAL>},org=(|h{pass=10})f{bal=<BBAL>}}}<SEPA>;,ole=<OLEA>,ose=<OSEP>},unc=n{sep=/(<TSTS>)?m{rat=0.7,vert=100,low=h{pass=10},asc=b{width=3,bnd=f{bal=<BBAL>},org=(|h{pass=10})f{bal=<BBAL>}}}<SEPA>;,ole=<OLEA>,ose=<OSEP>}}");
+
+  switch (flagval & (SCOTCH_STRATLEVELMIN | SCOTCH_STRATLEVELMAX)) {
+    case SCOTCH_STRATLEVELMIN :
+      tstsptr = "(levl<<LEVL>)|(vert>240)";
+      break;
+    case SCOTCH_STRATLEVELMAX :
+      tstsptr = "(levl<<LEVL>)&(vert>240)";
+      break;
+    case (SCOTCH_STRATLEVELMIN | SCOTCH_STRATLEVELMAX) :
+      tstsptr = "levl<<LEVL>";
+      break;
+    default :
+      tstsptr = "vert>240";
+      break;
+  }
+
+  sepaptr = ((flagval & SCOTCH_STRATSPEED) != 0)
+            ? ""
+            : "|m{rat=0.7,vert=100,low=h{pass=10},asc=b{width=3,bnd=f{bal=<BBAL>},org=(|h{pass=10})f{bal=<BBAL>}}}";
+
+  oleaptr = ((flagval & SCOTCH_STRATLEAFSIMPLE) != 0)
+            ? "s"
+            : "f{cmin=15,cmax=100000,frat=0.0}";
+
+  osepptr = ((flagval & SCOTCH_STRATSEPASIMPLE) != 0)
+            ? "s"
+            : "g";
+
+  stringSubst (bufftab, "<SEPA>", sepaptr);
+  stringSubst (bufftab, "<TSTS>", tstsptr);
+  stringSubst (bufftab, "<LEVL>", levltab);
+  stringSubst (bufftab, "<OLEA>", oleaptr);
+  stringSubst (bufftab, "<OSEP>", osepptr);
   stringSubst (bufftab, "<BBAL>", bbaltab);
 
   if (SCOTCH_stratGraphOrder (stratptr, bufftab) != 0) {
