@@ -1,4 +1,4 @@
-/* Copyright 2004,2007,2008,2010,2011 ENSEIRB, INRIA & CNRS
+/* Copyright 2004,2007,2008,2010-2012 IPB, Universite de Bordeaux, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -51,6 +51,8 @@
 /**                                 to   : 16 mar 2008     **/
 /**                # Version 5.1  : from : 01 jul 2010     **/
 /**                                 to   : 14 feb 2011     **/
+/**                # Version 6.0  : from : 01 jan 2012     **/
+/**                                 to   : 14 nov 2012     **/
 /**                                                        **/
 /************************************************************/
 
@@ -120,8 +122,6 @@ char *                      argv[])
 
   errorProg ("gord");
 
-  intRandInit ();
-
   if ((argc >= 2) && (argv[1][0] == '?')) {       /* If need for help */
     usagePrint (stdout, C_usageList);
     return     (0);
@@ -193,7 +193,7 @@ char *                      argv[])
           break;
         case 'V' :
           fprintf (stderr, "gord, version " SCOTCH_VERSION_STRING "\n");
-          fprintf (stderr, "Copyright 2004,2007,2008,2010,2011 ENSEIRB, INRIA & CNRS, France\n");
+          fprintf (stderr, "Copyright 2004,2007,2008,2010-2012 IPB, Universite de Bordeaux, INRIA & CNRS, France\n");
           fprintf (stderr, "This software is libre/free software under CeCILL-C -- see the user's manual for more information\n");
           return  (0);
         case 'v' :                               /* Output control info */
@@ -231,7 +231,7 @@ char *                      argv[])
     if (straptr != NULL)
       errorPrint ("main: options '-c' and '-o' are exclusive");
 
-    SCOTCH_stratGraphOrderBuild (&stradat, straval, 0.2);
+    SCOTCH_stratGraphOrderBuild (&stradat, straval, 0, 0.2);
   }
 
   clockStop  (&runtime[0]);                       /* Get input time */
