@@ -1,4 +1,4 @@
-/* Copyright 2004,2007,2008,2010,2011 ENSEIRB, INRIA & CNRS
+/* Copyright 2004,2007,2008,2010-2012 IPB, Universite de Bordeaux, INRIA & CNRS
 **
 ** This file is part of the Scotch software package for static mapping,
 ** graph partitioning and sparse matrix ordering.
@@ -44,6 +44,8 @@
 /**                                 to   : 16 mar 2008     **/
 /**                # Version 5.1  : from : 08 sep 2008     **/
 /**                                 to   : 14 feb 2011     **/
+/**                # Version 6.0  : from : 01 jan 2012     **/
+/**                                 to   : 14 nov 2012     **/
 /**                                                        **/
 /************************************************************/
 
@@ -111,8 +113,6 @@ char *                      argv[])
 
   errorProg ("mord");
 
-  intRandInit ();
-
   if ((argc >= 2) && (argv[1][0] == '?')) {       /* If need for help */
     usagePrint (stdout, C_usageList);
     return     (0);
@@ -183,7 +183,7 @@ char *                      argv[])
           break;
         case 'V' :
           fprintf (stderr, "mord, version " SCOTCH_VERSION_STRING "\n");
-          fprintf (stderr, "Copyright 2004,2007,2008,2010,2011 ENSEIRB, INRIA & CNRS, France\n");
+          fprintf (stderr, "Copyright 2004,2007,2008,2010-2012 IPB, Universite de Bordeaux, INRIA & CNRS, France\n");
           fprintf (stderr, "This software is libre/free software under CeCILL-C -- see the user's manual for more information\n");
           return  (0);
           break;
